@@ -15,8 +15,6 @@
    not, write to the Free Software Foundation, Inc., 51 Franklin Street,
    Suite 500, Boston, MA 02110-1335, USA. */
 
-#define _GNU_SOURCE
-
 #if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
@@ -69,6 +67,30 @@ static GDBM_FILE dbm;
 #define ypdb_close dbm_close
 static DBM *dbm;
 
+#elif defined (HAVE_LIBTC)
+
+#include <tcbdb.h>
+
+#define YPDB_REPLACE 1
+
+static TCBDB *dbm;
+
+static inline int
+ypdb_store(TCBDB *dbm, datum key, datum data, int mode)
+{
+  if (mode != YPDB_REPLACE)
+    return 1;
+
+  return !tcbdbput(dbm, key.dptr, key.dsize, data.dptr, data.dsize);
+}
+
+static inline void
+ypdb_close (TCBDB *dbm)
+{
+  tcbdbclose (dbm);
+  tcbdbdel (dbm);
+}
+
 #else
 
 #error "No database found or selected!"
@@ -130,6 +152,13 @@ create_file (char *fileName, char *dbmName, char *masterName,
   dbm = gdbm_open (filename, 0, GDBM_NEWDB | GDBM_FAST, 0600, NULL);
 #elif defined(HAVE_NDBM)
   dbm = dbm_open (filename, O_CREAT | O_RDWR, 0600);
+#elif defined(HAVE_LIBTC)
+  dbm = tcbdbnew();
+  if (!tcbdbopen(dbm, filename, BDBOWRITER | BDBOCREAT))
+  {
+    tcbdbdel(dbm);
+    dbm = NULL;
+  }
 #endif
   if (dbm == NULL)
     {
@@ -459,6 +488,9 @@ create_file (char *fileName, char *dbmName, char *masterName,
 #endif
 #else
   unlink (dbmName);
+#if defined(HAVE_LIBTC)
+	  chmod(filename, S_IRUSR|S_IWUSR);
+#endif
   rename (filename, dbmName);
 #endif
   free (filename);
@@ -475,10 +507,18 @@ dump_file (char *dbmName)
   dbm = gdbm_open (dbmName, 0, GDBM_READER, 0600, NULL);
 #elif defined(HAVE_NDBM)
   dbm = dbm_open (dbmName, O_RDONLY, 0600);
+#elif defined(HAVE_LIBTC)
+  dbm = tcbdbnew();
+  if (!tcbdbopen (dbm, dbmName, BDBOREADER | BDBONOLCK))
+  {
+    tcbdbdel(dbm);
+    dbm = NULL;
+  }
 #endif
   if (dbm == NULL)
     {
       fprintf (stderr, "makedbm: Cannot open %s\n", dbmName);
+      fprintf (stderr, "makedbm: Consider rebuilding maps using ypinit\n");
       exit (1);
     }
 #if defined(HAVE_COMPAT_LIBGDBM)
@@ -512,6 +552,32 @@ dump_file (char *dbmName)
 	      data.dsize, data.dptr);
       key = dbm_nextkey (dbm);
     }
+#elif defined(HAVE_LIBTC)
+  {
+    BDBCUR *cur;
+    cur = tcbdbcurnew (dbm);
+    if (tcbdbcurfirst (cur))
+      {
+        while ((key.dptr = tcbdbcurkey (cur, &key.dsize)) != NULL)
+          {
+            data.dptr = tcbdbcurval (cur, &data.dsize);
+            if (!data.dptr)
+	      {
+	        fprintf (stderr, "Error:\n");
+	        perror (dbmName);
+	        exit (1);
+              }
+            
+            printf ("%.*s\t%.*s\n",
+                  key.dsize, key.dptr,
+                  data.dsize, data.dptr);
+	  
+            if (!tcbdbcurnext (cur))
+              break;
+          }
+      }
+    tcbdbcurdel (cur);
+  }
 #endif
   ypdb_close (dbm);
 }
