@@ -12,10 +12,8 @@
 
    You should have received a copy of the GNU General Public
    License along with the YP Server; see the file COPYING. If
-   not, write to the Free Software Foundation, Inc., 675 Mass Ave,
-   Cambridge, MA 02139, USA. */
-
-#define _GNU_SOURCE
+   not, write to the Free Software Foundation, Inc., 51 Franklin Street,
+   Suite 500, Boston, MA 02110-1335, USA. */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -149,6 +147,12 @@ ypxfrd_getmap_1_svc (ypxfr_mapname *argp, struct svc_req *rqstp)
     if ((argp->xfr_db_type != XFR_DB_BSD_NDBM) &&
 	(argp->xfr_db_type != XFR_DB_ANY))
 #endif /* sun */
+#elif defined (HAVE_LIBQDBM)
+    if ((argp->xfr_db_type != XFR_DB_QDBM) &&
+	(argp->xfr_db_type != XFR_DB_ANY))
+#elif defined (HAVE_LIBTC)
+    if ((argp->xfr_db_type != XFR_DB_TC) &&
+	(argp->xfr_db_type != XFR_DB_ANY))
 #else
   if (argp->xfr_db_type != XFR_DB_ANY)
 #endif
