@@ -1,4 +1,4 @@
-/* Copyright (c) 1999, 2001, 2002 Thorsten Kukuk
+/* Copyright (c) 1999, 2001, 2002, 2011, 2013 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
@@ -12,10 +12,8 @@
 
    You should have received a copy of the GNU General Public
    License along with the YP Server; see the file COPYING. If
-   not, write to the Free Software Foundation, Inc., 675 Mass Ave,
-   Cambridge, MA 02139, USA. */
-
-#define _GNU_SOURCE
+   not, write to the Free Software Foundation, Inc., 51 Franklin Street,
+   Suite 500, Boston, MA 02110-1335, USA. */
 
 #if defined(HAVE_CONFIG_H)
 #include "config.h"
@@ -42,12 +40,19 @@
 #endif
 #if defined(HAVE_LIBGDBM)
 #include <gdbm.h>
+#elif defined(HAVE_LIBQDBM)
+#include <hovel.h>
 #elif defined(HAVE_NDBM)
 #include <ndbm.h>
+#elif defined(HAVE_LIBTC)
+#include <tcbdb.h>
 #endif
 #include "yp.h"
 #include "compat.h"
 #include <rpcsvc/ypclnt.h>
+#if defined(HAVE_RPC_CLNT_SOC_H)
+#include <rpc/clnt_soc.h> /* for clntudp_create() */
+#endif /* HAVE_RPC_CLNT_SOC_H */
 #include <arpa/nameser.h>
 #ifdef HAVE_SHADOW_H
 #include <shadow.h>
@@ -204,7 +209,7 @@ print_maps (char *server, char *domain)
 #if USE_FQDN
   struct hostent *hp = NULL;
 #endif
-  char *master, *domainname;
+  char *master = NULL, *domainname;
   struct ypmaplist *ypmap = NULL, *y, *old;
   int ret;
 
@@ -481,10 +486,12 @@ get_dbm_entry (char *key, char *map, char *domainname)
   static char mappath[MAXPATHLEN + 2];
   char *val;
   datum dkey, dval;
-#if defined(HAVE_LIBGDBM)
+#if defined(HAVE_COMPAT_LIBGDBM)
   GDBM_FILE dbm;
 #elif defined (HAVE_NDBM)
   DBM *dbm;
+#elif defined (HAVE_LIBTC)
+  TCBDB *dbm;
 #endif
 
   if (strlen (YPMAPDIR) + strlen (domainname) + strlen (map) + 3 < MAXPATHLEN)
@@ -495,23 +502,33 @@ get_dbm_entry (char *key, char *map, char *domainname)
       exit (1);
     }
 
-#if defined(HAVE_LIBGDBM)
+#if defined(HAVE_COMPAT_LIBGDBM)
   dbm = gdbm_open (mappath, 0, GDBM_READER, 0600, NULL);
 #elif defined(HAVE_NDBM)
-  dbm = dbm_open (mappath, O_CREAT | O_RDWR, 0600);
+  dbm = dbm_open (mappath, O_RDONLY, 0600);
+#elif defined(HAVE_LIBTC)
+  dbm = tcbdbnew();
+  if (!tcbdbopen(dbm, mappath, BDBOREADER | BDBONOLCK))
+    {
+      tcbdbdel(dbm);
+      dbm = NULL;
+    }
 #endif
   if (dbm == NULL)
     {
       fprintf (stderr, "yphelper: cannot open %s\n", mappath);
+      fprintf (stderr, "yphelper: consider rebuilding maps using ypinit\n");
       exit (1);
     }
 
   dkey.dptr = key;
   dkey.dsize = strlen (dkey.dptr);
-#if defined(HAVE_LIBGDBM)
+#if defined(HAVE_COMPAT_LIBGDBM)
   dval = gdbm_fetch (dbm, dkey);
 #elif defined(HAVE_NDBM)
   dval = dbm_fetch (dbm, dkey);
+#elif defined(HAVE_LIBTC)
+  dval.dptr = tcbdbget (dbm, dkey.dptr, dkey.dsize, &dval.dsize);
 #endif
   if (dval.dptr == NULL)
     val = NULL;
@@ -521,10 +538,13 @@ get_dbm_entry (char *key, char *map, char *domainname)
       strncpy (val, dval.dptr, dval.dsize);
       val[dval.dsize] = 0;
     }
-#if defined(HAVE_LIBGDBM)
+#if defined(HAVE_COMPAT_LIBGDBM)
   gdbm_close (dbm);
 #elif defined(HAVE_NDBM)
   dbm_close (dbm);
+#elif defined(HAVE_LIBTC)
+  tcbdbclose (dbm);
+  tcbdbdel (dbm);
 #endif
   return val;
 }
@@ -536,7 +556,7 @@ is_master (char *map, char *domain, char *host)
 #if USE_FQDN
   struct hostent *hp = NULL;
 #endif
-  char *hostname, *domainname;
+  char *hostname, *domainname, *val;
   int ret;
 
   if (domain != NULL)
@@ -577,7 +597,7 @@ is_master (char *map, char *domain, char *host)
 #endif
 
   if (strcasecmp (hostname,
-		  get_dbm_entry ("YP_MASTER_NAME", map, domainname)) == 0)
+	  (val = get_dbm_entry ("YP_MASTER_NAME", map, domainname))) == 0)
     ret = 0;
   else
     ret = 1;
