@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2001, 2002, 2003, 2005, 2006, 2009  Thorsten Kukuk
+/* Copyright (c) 2000, 2001, 2002, 2003, 2005, 2006, 2009, 2011, 2013  Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
@@ -12,15 +12,13 @@
 
    You should have received a copy of the GNU General Public
    License along with the YP Server; see the file COPYING. If
-   not, write to the Free Software Foundation, Inc., 675 Mass Ave,
-   Cambridge, MA 02139, USA. */
+   not, write to the Free Software Foundation, Inc., 51 Franklin Street,
+   Suite 500, Boston, MA 02110-1335, USA. */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#define _GNU_SOURCE
-
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -486,7 +484,7 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
   result->transid = argp->transid;
 
   valid = is_valid (rqstp, argp->map_parms.map, argp->map_parms.domain);
-  if (valid < 1)
+  if (valid < 1 && valid != -4) /* Map does not exist has a special meaning */
     {
       switch (valid)
 	{
@@ -522,9 +520,6 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
 		     inet_ntoa (rqhost->sin_addr));
 	  result->xfrstat = YPXFR_NODOM;
 	  break;
-	case -4:
-	  /* Map does not exist local, ignored, will be handled below */
-	  break;
 	}
       return TRUE;
     }
@@ -608,6 +603,15 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
 	  result->xfrstat = YPXFR_NODOM;
 	  return TRUE;
 	}
+      else
+	{
+	  if (debug_flag)
+	    log_msg ("\t->New map %s from %s (is a trusted master!)",
+		     argp->map_parms.map, argp->map_parms.peer);
+	  else
+	    log_msg ("New map %s from trusted master %s",
+		     argp->map_parms.map, inet_ntoa (rqhost->sin_addr));
+	}
     }
   /* If you wish to allow the transfer of new maps, change the next
      #if 1 statement to #if 0 */
@@ -616,9 +620,10 @@ ypproc_xfr_2_svc (ypreq_xfr *argp, ypresp_xfr *result,
     {
       /* We doesn't have the map, refuse the transfer */
       if (debug_flag)
-        log_msg ("\t->Ignored (I don't have this map)");
+        log_msg ("\t->Ignored (transfer %s from %s, map doesn't exist local)",
+		 argp->map_parms.map, inet_ntoa (rqhost->sin_addr));
       else
-        log_msg ("refuse to transfer %s from %s, map doesn't exist",
+        log_msg ("refuse to transfer %s from %s, map doesn't exist local",
 		 argp->map_parms.map, inet_ntoa (rqhost->sin_addr));
 
       result->xfrstat = YPXFR_REFUSED;
@@ -828,6 +833,7 @@ ypproc_all_2_svc (ypreq_nokey *argp, ypresp_all *result, struct svc_req *rqstp)
 	  if (debug_flag)
 	    log_msg ("\t-> Ignored (not a valid domain)");
 	  result->ypresp_all_u.val.stat = YP_NODOM;
+	  break;
 	case -4:
 	  if (debug_flag)
 	    log_msg ("\t-> Ignored (map does not exist)");
