@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005  Thorsten Kukuk
+/* Copyright (c) 2000-2005, 2011  Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    The YP Server is free software; you can redistribute it and/or
@@ -12,15 +12,13 @@
 
    You should have received a copy of the GNU General Public
    License along with the YP Server; see the file COPYING. If
-   not, write to the Free Software Foundation, Inc., 675 Mass Ave,
-   Cambridge, MA 02139, USA. */
+   not, write to the Free Software Foundation, Inc., 51 Franklin Street,
+   Suite 500, Boston, MA 02110-1335, USA. */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#define _GNU_SOURCE
-
 #include <unistd.h>
 #include <syslog.h>
 #if defined(HAVE_GETOPT_H)
@@ -51,8 +49,12 @@
 #endif /* HAVE_RPC_CLNT_SOC_H */
 #include "compat.h"
 
+#if defined(HAVE_COMPAT_LIBGDBM)
 #if defined(HAVE_LIBGDBM)
 #include <gdbm.h>
+#elif defined(HAVE_LIBQDBM)
+#include <hovel.h>
+#endif
 
 #define ypdb_store gdbm_store
 #define YPDB_REPLACE GDBM_REPLACE
@@ -67,6 +69,45 @@ static GDBM_FILE dbm;
 #define ypdb_close dbm_close
 #define ypdb_fetch dbm_fetch
 static DBM *dbm;
+#elif defined(HAVE_LIBTC)
+#include <tcbdb.h>
+
+#define YPDB_REPLACE 1
+
+static inline int
+ypdb_close (TCBDB *dbp)
+{
+  tcbdbclose (dbp);
+  tcbdbdel (dbp);
+  dbp = NULL;
+  return 0;
+}
+
+static int
+ypdb_store (TCBDB *dbm, datum key, datum data, int mode)
+{
+  if (mode != YPDB_REPLACE)
+    return 1;
+
+  return !tcbdbput(dbm, key.dptr, key.dsize, data.dptr, data.dsize);
+}
+
+static datum
+ypdb_fetch (TCBDB *bdb, datum key)
+{
+  datum res;
+  
+  if (!(res.dptr = tcbdbget(bdb, key.dptr, key.dsize, &res.dsize)))
+    {
+      res.dptr = NULL;
+      res.dsize = 0;
+    }
+  
+  return res;
+}
+
+static TCBDB *dbm;
+
 #endif
 
 #ifndef YPMAPDIR
@@ -246,6 +287,16 @@ ypxfrd_transfer (char *host, char *map, char *domain, char *tmpname)
   req.xfr_db_type = XFR_DB_BSD_NDBM;
   req.xfr_byte_order = XFR_ENDIAN_ANY;
 #endif
+#elif defined (HAVE_LIBQDBM)
+  req.xfr_db_type = XFR_DB_QDBM;
+#if defined(WORDS_BIGENDIAN)
+  req.xfr_byte_order = XFR_ENDIAN_BIG;
+#else
+  req.xfr_byte_order = XFR_ENDIAN_LITTLE;
+#endif
+#elif defined (HAVE_LIBTC)
+  req.xfr_db_type = XFR_DB_TC;
+  req.xfr_byte_order = XFR_ENDIAN_ANY;
 #endif
   memset (&resp, 0, sizeof (resp));
 
@@ -500,10 +551,17 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
       time_t localOrderNum = 0;
       datum inKey, inVal;
 
-#if defined(HAVE_LIBGDBM)
+#if defined(HAVE_COMPAT_LIBGDBM)
       dbm = gdbm_open (dbName_orig, 0, GDBM_READER, 0600, NULL);
 #elif defined(HAVE_NDBM)
-      dbm = dbm_open (dbName_orig, O_CREAT|O_RDWR, 0600);
+      dbm = dbm_open (dbName_orig, O_RDONLY, 0600);
+#elif defined(HAVE_LIBTC)
+      dbm = tcbdbnew ();
+      if (!tcbdbopen (dbm, dbName_orig, BDBOREADER | BDBONOLCK))
+        {
+          tcbdbdel (dbm);
+          dbm = NULL;
+        }
 #endif
       if (dbm == NULL)
         {
@@ -562,10 +620,17 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
       char orderNum[255];
       CLIENT *clnt_tcp;
 
-#if defined(HAVE_LIBGDBM)
+#if defined(HAVE_COMPAT_LIBGDBM)
       dbm = gdbm_open (dbName_temp, 0, GDBM_NEWDB, 0600, NULL);
 #elif defined(HAVE_NDBM)
       dbm = dbm_open (dbName_temp, O_CREAT|O_RDWR, 0600);
+#elif defined(HAVE_LIBTC)
+      dbm = tcbdbnew ();
+      if (!tcbdbopen (dbm, dbName_orig, BDBOWRITER | BDBOCREAT | BDBOTRUNC))
+        {
+          tcbdbdel (dbm);
+          dbm = NULL;
+        }
 #endif
       if (dbm == NULL)
         {
@@ -714,7 +779,12 @@ ypxfr (char *map, char *source_host, char *source_domain, char *target_domain,
     }
 
   if (result == 0)
-    rename (dbName_temp, dbName_orig);
+    {
+#if defined(HAVE_LIBTC)
+      chmod(dbName_temp, S_IRUSR|S_IWUSR);
+#endif
+      rename (dbName_temp, dbName_orig);
+    }
   else
     unlink(dbName_temp);
 
@@ -836,7 +906,11 @@ main (int argc, char **argv)
 
   if (target_domain == NULL)
     {
-      yp_get_default_domain (&target_domain);
+      if (yp_get_default_domain (&target_domain) != 0)
+	{
+	  log_msg ("Cannot get default domain");
+	  exit (1);
+	}
 
       if (target_domain == NULL)
 	{
